import java.util.HashMap;
import java.util.ArrayList;

public class TokenMarkov {
  protected HashMap<String, MarkovGram> grams;
  protected ArrayList<String> lineStarts;
  protected int n;
  protected int max;

  public TokenMarkov(int n_, int max_) {
    n = n_;
    max = max_;
    grams = new HashMap<String, MarkovGram>();
    lineStarts = new ArrayList<String>();
  }

  public void feedLine(String[] line) {

    // bail if length of array is less than n (otherwise we're liable to get
    // array indices that fall outside the array)
    if (line.length < n) {
      return;
    }

    // keep track of the first n elements of this line, to use as seeds in
    // the generation algorithm
    String lineStart = "";
    for (int i = 0; i < n; i++) {
      lineStart += line[i] + " ";
    }
    lineStarts.add(lineStart);

    // look at every group of n tokens in the current line and record
    // what token follows that group
    for (int i = 0; i < line.length - n; i++) {

      // construct upcoming gram as the current token appended to the next
      // n-1 tokens in the line
      String gramStr = "";
      for (int j = i; j < i+n; j++) {
        gramStr += line[j] + " ";
      }

      String next = line[i+n];

      // if we've already seen this gram, add a new next string
      if (grams.containsKey(gramStr)) {
        MarkovGram g = grams.get(gramStr);
        g.addNext(next);
      }
      // otherwise, create a new gram and store it
      else {
        MarkovGram g = new MarkovGram(gramStr);
        g.addNext(next);
        grams.put(gramStr, g);
      }
    }
    
  }

  public String generateLine() {

    int randomIndex = (int)(Math.random() * lineStarts.size());
    String current = lineStarts.get(randomIndex);

    String output = current;

    for (int i = 0; i < max; i++) {

      // if we can expand this gram, get a random string that can follow the
      // gram, and append it to the output
      if (grams.containsKey(current)) {
        MarkovGram g = grams.get(current);
        String nextStr = g.getRandomNext();
        output += nextStr + " ";

        // construct next key as the last n tokens from the output
        String[] currentTokens = output.split(" ");
        current = "";
        for (int j = currentTokens.length - n; j < currentTokens.length; j++) {
          current += currentTokens[j] + " ";
        }

      }
      // if we can't find this gram, then the party's over: stop generation!
      else {
        break;
      }
    }
    return output;

  }

  public void dump() {
    for (MarkovGram m: grams.values()) {
      m.dump();
    }
  }

}
