import com.decontextualize.a2z.TextFilter;
import rita.*;
import rita.wordnet.*;

public class WordNetRewrite extends TextFilter {
  public static void main(String[] args) {
    new WordNetRewrite().run();
  }

  protected RiWordnet wordnet = new RiWordnet(null);

  public void eachLine(String line) {
    String[] tokens = RiTa.tokenize(line);
    for (String token: tokens) {

      // attempt to determine the part of speech for this token
      String pos = wordnet.getBestPos(token);

      // if it's something that wordnet can deal with, then...
      if (token.length() >= 4 && pos != null) {

        double rand = Math.random();

        if (rand < 0.25) {
          // attempt to print a random antonym
          String[] antonyms = wordnet.getAntonyms(token, pos);
          if (antonyms != null) {
            print(choice(antonyms) + " ");
            continue;
          }
        }
        else if (rand < 0.5) {
          // attempt to print a random holonym (something that has this as a
          // part - inverse of meronym)
          String[] holonyms = wordnet.getHolonyms(token, pos);
          if (holonyms != null) {
            print(choice(holonyms) + " ");
            continue;
          }
        }
        else if (rand < 0.75) {
          // attempt to print a random hypernym
          String[] hypernyms = wordnet.getHypernyms(token, pos);
          if (hypernyms != null) {
            print(choice(hypernyms) + " ");
            continue;
          }
        }
        else {
          // 1 in 4 chance of printing a random word
          print(wordnet.getRandomWord(pos) + " ");
          continue;
        }

      }

      // print the token if none of the above succeeded
      print(token + " ");

    }
    println();
  }

  public static String choice(String[] options) {
    int index = (int)(Math.random() * options.length);
    return options[index];
  }

}
